#include "e3bot-actuators.hpp"
#include <Arduino.h>
#include "util.hpp"

e3bot::Actuators::Actuators(const PinDef &pinDef) :
  _pinDef(pinDef),
	_inputFwd(0.0), _inputTurn(0.0), _inputHead(0.0)
{
  for (int i = 0; i < SERVOS_COUNT; ++i)
  {
    _servos[i] = Servo(i);
  }

  tryAttach(pinDef.pinWheelL);
  tryAttach(pinDef.pinWheelR);
  tryAttach(pinDef.pinHeadL);
  tryAttach(pinDef.pinHeadR);
	
	setInput(0.0, 0.0, 0.0);
}

e3bot::Actuators::~Actuators()
{
}

void e3bot::Actuators::setInput(double inputFwd, double inputTurn, double inputHead)
{
	_inputFwd = inputFwd;
	_inputTurn = inputTurn;
	_inputHead = inputHead;
	
	// Process wheels
	double maxWheelSpeed = e3bot::abs(_inputFwd) + e3bot::abs(_inputTurn);
	if (maxWheelSpeed > 1.0)
	{
		_inputFwd /= maxWheelSpeed;
		_inputTurn /= maxWheelSpeed;
	}

  Servo *svWheelL = tryGetServo(_pinDef.pinWheelL);
	if (svWheelL) {svWheelL->write(90.0 + 90.0 * (+_inputFwd -_inputTurn));}
 
  Servo *svWheelR = tryGetServo(_pinDef.pinWheelR);
	if (svWheelL) {svWheelR->write(90.0 + 90.0 * (-_inputFwd -_inputTurn));}
	
	// Process head
	_inputHead = e3bot::clamp(_inputHead, -1.0, 1.0);
}

void e3bot::Actuators::setServo(int8_t index, double value)
{
  Servo *servo = tryGetServo(index);
  
  if (!servo) { return; }

  servo->attach();
  servo->write(90.0 + 90.0 * value);
}

void e3bot::Actuators::updateServos(unsigned long deltaTimeMS)
{
  double deltaTime = deltaTimeMS * 0.001;
  
  double nextInputHead = _inputHead;
  double moveSpeed = 0.0;
  if (_inputHead < 0.0) {
    nextInputHead = e3bot::min(0.0, _inputHead + deltaTime);
    moveSpeed = e3bot::max(_inputHead / deltaTime, -1.0) * 90.0;
  } else {
    nextInputHead = e3bot::max(0.0, _inputHead - deltaTime);
    moveSpeed = e3bot::min(_inputHead / deltaTime, 1.0) * 90.0;
  }
  
  Servo *svHeadL = tryGetServo(_pinDef.pinHeadL);
  if (svHeadL) {svHeadL->write(90.0 - moveSpeed);}
  
  Servo *svHeadR = tryGetServo(_pinDef.pinHeadR);
  if (svHeadR) {svHeadR->write(90.0 + moveSpeed);}

  _inputHead = nextInputHead;
}

void e3bot::Actuators::reset()
{
  for (int i = 0; i < SERVOS_COUNT; ++i)
  {
    if (i == _pinDef.pinWheelL) {continue;}
    if (i == _pinDef.pinWheelR) {continue;}
    if (i == _pinDef.pinHeadL) {continue;}
    if (i == _pinDef.pinHeadR) {continue;}
    
    _servos[i].detach();
  }

  setInput(0.0, 0.0, 0.0);
}
