#pragma once
#include <cstdint>
#include "esp32-servo.hpp"

#define SERVOS_COUNT 32

namespace e3bot
{
  class Actuators
  {
  public:
  	struct PinDef
  	{
  		uint8_t pinWheelL;
  		uint8_t pinWheelR;
  		uint8_t pinHeadL;
  		uint8_t pinHeadR;
  	};
  	Actuators(const PinDef &pinDef);
  	~Actuators();
  	
  	void setInput(double inputFwd, double inputTurn, double inputHead);
    void setServo(int8_t index, double value);
    void updateServos(unsigned long deltaTimeMS);
    void reset();
  	
  private:
  	Servo _servos[SERVOS_COUNT];
    PinDef _pinDef;

    inline Servo *tryGetServo(int pin) {
      if (pin < 0 || pin >= SERVOS_COUNT) { return nullptr; }
      return _servos + pin;
    }

    inline void tryAttach(int pin) {
      Servo *servo = tryGetServo(pin);
      if (servo) { servo->attach(); }
    }
  	
  	// All inputs range from -1.0 to 1.0, inclusive.
  	double _inputFwd;  // Along the X axis. Forward is +ve, backward is -ve.
  	double _inputTurn; // About the Z axis. Anticlockwise is +ve, clockwise is -ve.
  	double _inputHead; // About the Y axis. Tilting down is +ve, tilting up is -ve.
  };
}
