#include "esp32-servo.hpp"
#include "util.hpp"
#include <cmath>
#include <Arduino.h>
#include <esp32-hal-ledc.h>

const uint8_t PIN_NONE = -1;
const uint8_t CHANNEL_NONE = -1;
const int CHANNEL_COUNT = 16;

const double PWM_FREQ = 50.0;
const uint8_t PWM_WIDTH = 16;

const double PULSE_DURATION_0 = 0.0005;
const double PULSE_DURATION_180 = 0.0025;

const double SERVO_DUTY_OFFSET = (1 << PWM_WIDTH) * PULSE_DURATION_0 * PWM_FREQ; // 1638.4
const double SERVO_DUTY_DEGREE_DELTA = // 36.409
	(1 << PWM_WIDTH) * PWM_FREQ *
	(PULSE_DURATION_180 - PULSE_DURATION_0) /
	(180.0);
const double DEAD_ZONE_LOWER = 88.0;
const double DEAD_ZONE_UPPER = 92.0;
static uint32_t _channelMap = 0;

Servo::Servo() :
  _pin(PIN_NONE),
  _channel(CHANNEL_NONE)
{
}

Servo::Servo(uint8_t pin) :
	_pin(pin),
	_channel(CHANNEL_NONE)
{
}

Servo::~Servo() {
	detach();
}

bool Servo::attach()
{
	if (_channel != CHANNEL_NONE) { return true; }
	
	for (_channel = 0; _channel < CHANNEL_COUNT; ++_channel)
	{
		uint32_t channelMask = 1 << _channel;
		if ((_channelMap & channelMask) == 0)
		{
			_channelMap |= channelMask;
			break;
		}
	}
	
	if (_channel >= CHANNEL_COUNT)
	{
		_channel = CHANNEL_NONE;
		return false;
	}
	
	ledcSetup(_channel, PWM_FREQ, PWM_WIDTH);
	ledcAttachPin(_pin, _channel);
	
	return true;
}

void Servo::detach()
{
	if (_channel != CHANNEL_NONE)
	{
    ledcDetachPin(_pin);
		_channelMap &= ~(1 << _channel);
		_channel = CHANNEL_NONE;
	}
}

bool Servo::write(double value)
{
 if (value > DEAD_ZONE_LOWER && value < DEAD_ZONE_UPPER) {
  detach();
  return false;
 } else {
  attach();
	
	value = e3bot::clamp(value, 0.0, 180.0);
	uint32_t duty = (uint32_t)(SERVO_DUTY_OFFSET + SERVO_DUTY_DEGREE_DELTA * value);
	
	ledcWrite(_channel, duty);
	return true;
 }
}
